@@ -77,7 +77,9 @@ typedef enum {
     hmat_factorization_none = -1,
     hmat_factorization_lu,
     hmat_factorization_ldlt,
-    hmat_factorization_llt
+    hmat_factorization_llt,
+    hmat_factorization_hodlr,
+    hmat_factorization_hodlrsym
 } hmat_factorization_t;
 
 typedef struct hmat_block_info_struct {
@@ -310,6 +312,8 @@ typedef struct {
     double ratio;
     /** maximum block width */
     size_t max_width;
+    /** A scale factor (default to -1 for "unspecified" */
+    double scale;
 } hmat_admissibility_param_t;
 
 /** Init an hmat_admissibility_param structure with default values */
@@ -327,6 +331,13 @@ HMAT_API void hmat_update_admissibility(hmat_admissibility_t*, hmat_admissibilit
 /* Create a standard (Hackbusch) admissibility condition, with a given eta */
 HMAT_API hmat_admissibility_t* hmat_create_admissibility_standard(double eta);
 
+/**
+ * @brief Create an admissibility which will generate a HODLR matrix.
+ *
+ * All blocks except diagonal ones are admissible.
+ */
+HMAT_API hmat_admissibility_t* hmat_create_admissibility_hodlr(void);
+
 /**
  * @brief Create an admissibility condiction which set all blocks as admissible
  * @param max_block_size The maximum acceptable block size in number of values (rows * cols)
@@ -671,6 +682,10 @@ typedef struct
       \return the norm
     */
     double (*norm)(hmat_matrix_t* hmatrix);
+
+    /*! \brief Compute the logarithm of the determinante of a HMatrix. */
+    int (*logdet)(hmat_matrix_t* hmatrix, double * result);
+
     /*! \brief Inverse a HMatrix in place.
 
       \param hmatrix the matrix to inverse
@@ -787,8 +802,8 @@ typedef struct
       \param hmatrix_c
       \return 0 for success
     */
-    int (*gemm)(char trans_a, char trans_b, void* alpha, hmat_matrix_t* hmatrix,
-                     hmat_matrix_t* hmatrix_b, void* beta, hmat_matrix_t* hmatrix_c);
+    int (*gemm)(char trans_a, char trans_b, const void* alpha, hmat_matrix_t* hmatrix,
+                hmat_matrix_t* hmatrix_b, const void* beta, hmat_matrix_t* hmatrix_c);
 
     /*! \brief y := y + a * x */
     int (*axpy)(void* a, hmat_matrix_t* x, hmat_matrix_t* hmatrix_y);
@@ -850,8 +865,8 @@ typedef struct
       \param nrhs
       \return 0 for success
     */
-    int (*gemm_dense)(char trans_b, char trans_x, char side, void* alpha, hmat_matrix_t* holder,
-                      void* vec_x, void* beta, void* vec_y, int nrhs);
+    int (*gemm_dense)(char trans_b, char trans_x, char side, const void* alpha, hmat_matrix_t* holder,
+                      void* vec_x, const void* beta, void* vec_y, int nrhs);
     /*! \brief A <- A + alpha Id
 
       \param hmatrix
