@@ -34,6 +34,7 @@
 
 namespace trace {
 
+  /* \brief pointer to a function return the worker Id (between 0 and nbWorkers-1) or -1 in a sequential section */
   int (*nodeIndexFunction)() = NULL;
 
   /** \brief Set the function used to get the root index.
@@ -41,7 +42,7 @@ namespace trace {
       \return a number between 0 (not in a parallel region) and the number of
       workers (included).
    */
-  static int currentNodeIndex() {
+  int currentNodeIndex() {
     int res = (nodeIndexFunction ? nodeIndexFunction() : -1) + 1;
     assert(res>=0 && res<MAX_ROOTS);
     return res;
@@ -56,7 +57,7 @@ namespace trace {
   void* Node::enclosingContext[MAX_ROOTS] = {};
 
   Node::Node(const char* _name, Node* _parent)
-    : name(_name), data(), parent(_parent), children() {}
+    : name_(_name), data(), parent(_parent), children() {}
 
   Node::~Node() {
     for (std::vector<Node*>::iterator it = children.begin(); it != children.end(); ++it) {
@@ -124,7 +125,7 @@ namespace trace {
       // On cherche la correspondance avec le pointeur. Puisqu'on demande que
       // tous les noms soient des pointeurs qui existent tout le long de
       // l'execution, on peut forcer l'unicite.
-      if ((*it)->name == name) {
+      if ((*it)->name_ == name) {
 	return *it;
       }
     }
@@ -133,7 +134,7 @@ namespace trace {
 
   void Node::jsonDump(std::ofstream& f) const {
     f << "{"
-      << "\"name\": \"" << name << "\", "
+      << "\"name\": \"" << name_ << "\", "
       << "\"id\": \"" << this << "\", "
       << "\"n\": " << data.n << ", "
       << "\"totalTime\": " << data.totalTime / 1e9 << ", "
@@ -178,12 +179,13 @@ namespace trace {
     UM_NS::unordered_map<void*, Node*>::iterator it = currentNodes[index].find(enclosing);
     Node* current;
     if (it == currentNodes[index].end()) {
-      // TODO : avec runtime, les threads N+1 et N+2 ne sont pas des workers, ce sont les threads MPI et IO
+      // TODO : avec toyrt, les threads 1 et 2 ne sont pas des workers, ce sont les threads IO & MPI
+      // Il faudrait que le code appelant donne le nom du noeud plutot qu'un index
       char *name = const_cast<char*>("root");
       if (index != 0) {
         name = strdup("Worker #XXX - 0xXXXXXXXXXXXXXXXX"); // Worker ID - enclosing
         assert(name);
-        sprintf(name, "Worker #%03d - %p", index, enclosing);
+        sprintf(name, "Worker #%03d - %p", index, enclosing); // Recuperer le nom de cet enclosing !
       }
       current = new Node(name, NULL);
       currentNodes[index][enclosing] = current;
