@@ -31,7 +31,7 @@
 #include <vector>
 #include <cfloat>
 #include <cstring>
-
+#include <limits>
 #include "cluster_tree.hpp"
 #include "assembly.hpp"
 #include "rk_matrix.hpp"
@@ -60,6 +60,23 @@ using std::vector;
 using std::min;
 using std::max;
 
+namespace {
+struct EnvVar {
+  /** */
+  int logAcaPartialMinSize;
+  EnvVar() {
+	// Enable ACA partial verbose mode for blocks larger than a given size.
+    const char * logAcaStr = getenv("HMAT_LOG_ACA_PARTIAL");
+    if(logAcaStr == nullptr) {
+      logAcaPartialMinSize = std::numeric_limits<int>::max();
+    } else {
+      logAcaPartialMinSize = atoi(logAcaStr);
+    }
+  }
+};
+static const EnvVar env;
+} // namespace
+
 namespace hmat {
 
 /** \brief Updates a row to reflect its current value in the matrix.
@@ -74,7 +91,8 @@ template<typename T>
 static void updateRow(Vector<T>& rowVec, int row, const vector<Vector<T>*>& rows,
                       const vector<Vector<T>*>& cols, int k) {
   for (int l = 0; l < k; l++) {
-    rowVec.axpy(Constants<T>::mone * (*cols[l])[row], rows[l]);
+    assert(row < cols[l]->rows);
+    rowVec.axpy(-(*cols[l])[row], rows[l]);
   }
 }
 
@@ -85,19 +103,19 @@ static void updateCol(Vector<T>& colVec, int col, const vector<Vector<T>*>& cols
 }
 
 
-template<typename T> static void findMax(const FullMatrix<T>* m, int& i, int& j) {
-  if (m->data.lda == m->data.rows) {
+template<typename T> static void findMax(const ScalarArray<T>& m, int& i, int& j) {
+  if (m.lda == m.rows) {
     // quick path
-    int k = proxy_cblas::i_amax(m->data.rows*m->data.cols, m->data.const_ptr(), 1);
-    i = k % m->data.rows ;
-    j = (k - i) / m->data.rows ;
+    int k = proxy_cblas::i_amax(m.rows*m.cols, m.const_ptr(), 1);
+    i = k % m.rows ;
+    j = (k - i) / m.rows ;
   } else {
     i = 0;
     j = 0;
     double maxNorm = 0.;
-    for (int col = 0; col < m->cols(); col++) {
-      int row = proxy_cblas::i_amax(m->data.rows, m->data.const_ptr(0,col), 1);
-      const double norm = squaredNorm<T>(m->get(row, col));
+    for (int col = 0; col < m.cols; col++) {
+      int row = proxy_cblas::i_amax(m.rows, m.const_ptr(0,col), 1);
+      const double norm = squaredNorm<T>(m.get(row, col));
       if (norm > maxNorm) {
         i = row;
         j = col;
@@ -258,43 +276,37 @@ CompressionSVD::compress(const ClusterAssemblyFunction<Z_t>& block) const {
     return doCompressionSVD<Z_t>(block, epsilon_);
 }
 
-
 template<typename T>
-RkMatrix<typename Types<T>::dp>*
-doCompressionAcaFull(const ClusterAssemblyFunction<T>& block, double compressionEpsilon) {
+void acaFull(ScalarArray<T> & m, ScalarArray<T>* & tmpA, ScalarArray<T>* & tmpB, double compressionEpsilon) {
   DECLARE_CONTEXT;
-  typedef typename Types<T>::dp dp_t;
-  // TODO: use ClusterAssemblyFunction to optimize with blockinfo_t
-  FullMatrix<dp_t>* m = block.assemble();
-
   double estimateSquaredNorm = 0;
-  int maxK = min(m->rows(), m->cols());
+  int maxK = min(m.rows, m.cols);
 
-  ScalarArray<dp_t> tmpA(m->rows(), maxK);
-  ScalarArray<dp_t> tmpB(m->cols(), maxK);
+  tmpA = new ScalarArray<T>(m.rows, maxK);
+  tmpB = new ScalarArray<T>(m.cols, maxK);
   int nu;
 
   for (nu = 0; nu < maxK; nu++) {
     int i_nu, j_nu;
     findMax(m, i_nu, j_nu);
-    const dp_t delta = m->get(i_nu, j_nu);
+    const T delta = m.get(i_nu, j_nu);
     if (squaredNorm(delta) == 0.) {
       break;
     }
 
     // Creation of the vectors A_i_nu and B_j_nu
     {
-      Vector<dp_t> va_nu(tmpA, nu);
-      Vector<dp_t> vb_nu(tmpB, nu);
+      Vector<T> va_nu(*tmpA, nu);
+      Vector<T> vb_nu(*tmpB, nu);
 
-      for (int i = 0; i < m->rows(); i++)
-        va_nu[i] = m->get(i, j_nu);
-      for (int j = 0; j < m->cols(); j++)
-        vb_nu[j] = m->get(i_nu, j) / delta;
+      for (int i = 0; i < m.rows; i++)
+        va_nu[i] = m.get(i, j_nu);
+      for (int j = 0; j < m.cols; j++)
+        vb_nu[j] = m.get(i_nu, j) / delta;
 
       // performs the rank 1 operation m := m - va_nu*vb_nu^T
       // in order to nullify m->get(i_nu, j_nu) (the previous maximum value)
-      m->data.rankOneUpdate(Constants<dp_t>::mone, va_nu, vb_nu);
+      m.rankOneUpdate(-1, va_nu, vb_nu);
 
       // Update the estimate norm
       // Let S_{k-1} be the previous estimate. We have (for the Frobenius norm):
@@ -303,9 +315,9 @@ doCompressionAcaFull(const ClusterAssemblyFunction<T>& block, double compression
       // The sum
       double newEstimate = 0.0;
       for (int l = 0; l < nu - 1; l++) {
-        Vector<dp_t> a_l(tmpA, l);
-        Vector<dp_t> b_l(tmpB, l);
-        newEstimate += hmat::real(Vector<dp_t>::dot(&va_nu, &a_l) * Vector<dp_t>::dot(&vb_nu, &b_l));
+        Vector<T> a_l(*tmpA, l);
+        Vector<T> b_l(*tmpB, l);
+        newEstimate += hmat::real(Vector<T>::dot(&va_nu, &a_l) * Vector<T>::dot(&vb_nu, &b_l));
       }
       estimateSquaredNorm += 2.0 * newEstimate;
       const double a_nu_norm_2 = va_nu.normSqr();
@@ -321,33 +333,50 @@ doCompressionAcaFull(const ClusterAssemblyFunction<T>& block, double compression
       }
     }
   }
-  delete m;
-
   if (nu == 0) {
-    return new RkMatrix<dp_t>(NULL, block.rows, NULL, block.cols);
+    delete tmpA;
+    delete tmpB;
+    tmpA = nullptr;
+    tmpB = nullptr;
+  } else {
+    tmpA->resize(nu);
+    tmpB->resize(nu);
   }
+}
 
-  tmpA.cols=nu; // resize tmpA and tmpB for the copy
-  tmpB.cols=nu;
+template<typename T> RkMatrix<T>* acaFull(FullMatrix<T>* m, double compressionEpsilon) {
+  DECLARE_CONTEXT;
+  ScalarArray<T> * tmpA;
+  ScalarArray<T> * tmpB;
+  acaFull(m->data, tmpA, tmpB, compressionEpsilon);
+  auto rows = m->rows_;
+  auto cols = m->cols_;
+  return new RkMatrix<T>(tmpA, rows, tmpB, cols);
+}
 
-  return new RkMatrix<dp_t>(tmpA.copy(), block.rows, tmpB.copy(), block.cols);
+template<typename T> RkMatrix<typename Types<T>::dp>*
+doCompressionAcaFull(const ClusterAssemblyFunction<T>& block, double eps) {
+  FullMatrix<typename Types<T>::dp> * m = block.assemble();
+  auto r = acaFull<typename Types<T>::dp>(m, eps);
+  delete m;
+  return r;
 }
 
 RkMatrix<Types<S_t>::dp>*
 CompressionAcaFull::compress(const ClusterAssemblyFunction<S_t>& block) const {
-    return doCompressionAcaFull<S_t>(block, epsilon_);
+  return doCompressionAcaFull(block, epsilon_);
 }
 RkMatrix<Types<D_t>::dp>*
 CompressionAcaFull::compress(const ClusterAssemblyFunction<D_t>& block) const {
-    return doCompressionAcaFull<D_t>(block, epsilon_);
+  return doCompressionAcaFull(block, epsilon_);
 }
 RkMatrix<Types<C_t>::dp>*
 CompressionAcaFull::compress(const ClusterAssemblyFunction<C_t>& block) const {
-    return doCompressionAcaFull<C_t>(block, epsilon_);
+  return doCompressionAcaFull(block, epsilon_);
 }
 RkMatrix<Types<Z_t>::dp>*
 CompressionAcaFull::compress(const ClusterAssemblyFunction<Z_t>& block) const {
-    return doCompressionAcaFull<Z_t>(block, epsilon_);
+  return doCompressionAcaFull(block, epsilon_);
 }
 
 template<typename T>
@@ -360,6 +389,7 @@ doCompressionAcaPartial(const ClusterAssemblyFunction<T>& block, double compress
   const int rowCount = block.rows->size();
   const int colCount = block.cols->size();
   int maxK = min(rowCount, colCount);
+  bool verbose = maxK > env.logAcaPartialMinSize;
   // Contains false for the rows that were already used as pivot
   vector<bool> rowFree(rowCount, true);
   int rowPivotCount = 0;
@@ -368,18 +398,19 @@ doCompressionAcaPartial(const ClusterAssemblyFunction<T>& block, double compress
   vector<Vector<dp_t>*> aCols;
   vector<Vector<dp_t>*> bCols;
 
-  int I = 0;
+  int row_index = 0;
   int J = 0;
   int k = 0;
 
   RandomPivotManager<T> randomPivotManager(block, useRandomPivots ? max(rowCount, colCount) : 0);
-
+  if(verbose)
+    printf("[HMat] Starting ACA Partial on %sx%s\n", block.rows->description().c_str(), block.cols->description().c_str());
   do {
     Vector<dp_t>* bCol = new Vector<dp_t>(block.cols->size());
     // Calculation of row I and its residue
-    block.getRow(I, *bCol);
-    updateRow(*bCol, I, bCols, aCols, k);
-    rowFree[I] = false;
+    block.getRow(row_index, *bCol);
+    updateRow(*bCol, row_index, bCols, aCols, k);
+    rowFree[row_index] = false;
 
     // Find max and argmax of the residue
     double maxNorm2 = 0.;
@@ -392,22 +423,22 @@ doCompressionAcaPartial(const ClusterAssemblyFunction<T>& block, double compress
     }
 
     Pivot<dp_t > randomOrDefaultPivot = randomPivotManager.GetPivot();
-    if(I!=randomOrDefaultPivot.row_ && squaredNorm(randomOrDefaultPivot.value_) > maxNorm2){
-      I = randomOrDefaultPivot.row_;
+    if(row_index!=randomOrDefaultPivot.row_ && squaredNorm(randomOrDefaultPivot.value_) > maxNorm2){
+      row_index = randomOrDefaultPivot.row_;
       delete bCol;
       continue;
     }
 
-    if ((*bCol)[J] == Constants<dp_t>::zero) {
+    if ((*bCol)[J] == dp_t(0)) {
       delete bCol;
       // We look for another row which has not already been used.
-      I = 0;
-      while (!rowFree[I]) {
-        I++;
+      row_index = 0;
+      while (!rowFree[row_index]) {
+        row_index++;
       }
     } else {
       // Find pivot and scale column B
-      dp_t pivot = Constants<dp_t>::pone / (*bCol)[J];
+      dp_t pivot = 1. / (*bCol)[J];
       bCol->scale(pivot);
       bCols.push_back(bCol);
 
@@ -415,7 +446,7 @@ doCompressionAcaPartial(const ClusterAssemblyFunction<T>& block, double compress
       Vector<dp_t>* aCol = new Vector<dp_t>(block.rows->size());
       block.getCol(J, *aCol);
       updateCol(*aCol, J, aCols, bCols, k);
-      randomPivotManager.AddUsedPivot(bCol, aCol, I, J);
+      randomPivotManager.AddUsedPivot(bCol, aCol, row_index, J);
       colFree[J] = false;
       aCols.push_back(aCol);
 
@@ -425,7 +456,7 @@ doCompressionAcaPartial(const ClusterAssemblyFunction<T>& block, double compress
         const double norm2 = squaredNorm<dp_t>((*aCol)[i]);
         if (rowFree[i] && norm2 > maxNorm2) {
           maxNorm2 = norm2;
-          I = i;
+          row_index = i;
         }
       }
 
@@ -450,9 +481,13 @@ doCompressionAcaPartial(const ClusterAssemblyFunction<T>& block, double compress
       if (ab_norm_2 < compressionEpsilon * compressionEpsilon * estimateSquaredNorm) {
         break;
       }
+      if(verbose) {
+        printf("%d %g\n", rowPivotCount, sqrt(ab_norm_2/estimateSquaredNorm));
+        fflush(stdout);
+      }
     }
     rowPivotCount++;
-  } while (rowPivotCount < maxK);
+  } while (rowPivotCount < maxK && row_index < rowCount);
 
   ScalarArray<dp_t> *newA, *newB;
   if (k != 0) {
@@ -545,7 +580,7 @@ doCompressionAcaPlus(const ClusterAssemblyFunction<T>& block, double compression
       // Calculate a
       block.getCol(j_star, *aVec);
       updateCol<dp_t>(*aVec, j_star, aCols, bCols, k);
-      if(pivot != Constants<dp_t>::zero) aVec->scale(Constants<dp_t>::pone / pivot);
+      if(pivot != dp_t(0)) aVec->scale(1. / pivot);
     } else {
       // j_star is fixed, we look for i_star
       block.getCol(j_star, *aVec);
@@ -555,7 +590,7 @@ doCompressionAcaPlus(const ClusterAssemblyFunction<T>& block, double compression
       // Calculate b
       block.getRow(i_star, *bVec);
       updateRow<dp_t>(*bVec, i_star, bCols, aCols, k);
-      if(pivot != Constants<dp_t>::zero) bVec->scale(Constants<dp_t>::pone / pivot);
+      if(pivot != dp_t(0)) bVec->scale(1. / pivot);
     }
 
     rowFree[i_star] = false;
@@ -586,8 +621,8 @@ doCompressionAcaPlus(const ClusterAssemblyFunction<T>& block, double compression
     }
 
     // Update of a_ref and b_ref
-    aRef.axpy(Constants<dp_t>::mone * (*bCols[k - 1])[j_ref], aCols[k - 1]);
-    bRef.axpy(Constants<dp_t>::mone * (*aCols[k - 1])[i_ref], bCols[k - 1]);
+    aRef.axpy(-(*bCols[k - 1])[j_ref], aCols[k - 1]);
+    bRef.axpy(-(*aCols[k - 1])[i_ref], bCols[k - 1]);
     const bool needNewA = aRef.isZero() || (j_star == j_ref);
     const bool needNewB = bRef.isZero() || (i_star == i_ref);
 
@@ -688,7 +723,7 @@ RkMatrix<typename Types<T>::dp>* compress(
             // Pass a negative value to tell formattedAddParts to not call truncate()
             // FIXME: investigate why calling truncate from formattedAddParts or here
             //        gives different results
-            rk->formattedAddParts(-epsilon, &Constants<dp_t>::pone, &stratumRk, 1);
+            rk->axpy(-epsilon, 1, stratumRk);
             rk->truncate(epsilon);
         }
         delete stratumRk;
@@ -717,7 +752,7 @@ template<typename T> RkMatrix<typename Types<T>::dp>* compressOneStratum(
       HMAT_ASSERT(false);
     }
 
-    rkFull->axpy(Constants<T>::mone, full);
+    rkFull->axpy(-1, full);
     double diffNorm = rkFull->norm();
     if (diffNorm > HMatrix<T>::validationErrorThreshold * fullNorm ) {
       std::cout << block.rows->description() << "x" << block.cols->description() << std::endl
@@ -762,6 +797,16 @@ template RkMatrix<D_t>* truncatedSvd(FullMatrix<D_t>* m, double eps);
 template RkMatrix<C_t>* truncatedSvd(FullMatrix<C_t>* m, double eps);
 template RkMatrix<Z_t>* truncatedSvd(FullMatrix<Z_t>* m, double eps);
 
+template RkMatrix<S_t>* acaFull(FullMatrix<S_t>*, double);
+template RkMatrix<D_t>* acaFull(FullMatrix<D_t>*, double);
+template RkMatrix<C_t>* acaFull(FullMatrix<C_t>*, double);
+template RkMatrix<Z_t>* acaFull(FullMatrix<Z_t>*, double);
+
+template void acaFull(ScalarArray<S_t> &, ScalarArray<S_t>* &, ScalarArray<S_t>* &, double);
+template void acaFull(ScalarArray<D_t> &, ScalarArray<D_t>* &, ScalarArray<D_t>* &, double);
+template void acaFull(ScalarArray<C_t> &, ScalarArray<C_t>* &, ScalarArray<C_t>* &, double);
+template void acaFull(ScalarArray<Z_t> &, ScalarArray<Z_t>* &, ScalarArray<Z_t>* &, double);
+
 template RkMatrix<Types<S_t>::dp>* compress<S_t>(const CompressionAlgorithm* method, const Function<S_t>& f, const ClusterData* rows, const ClusterData* cols, double epsilon, const AllocationObserver &);
 template RkMatrix<Types<D_t>::dp>* compress<D_t>(const CompressionAlgorithm* method, const Function<D_t>& f, const ClusterData* rows, const ClusterData* cols, double epsilon, const AllocationObserver &);
 template RkMatrix<Types<C_t>::dp>* compress<C_t>(const CompressionAlgorithm* method, const Function<C_t>& f, const ClusterData* rows, const ClusterData* cols, double epsilon, const AllocationObserver &);
