@@ -25,13 +25,6 @@
   \brief Dense Matrix implementation.
 */
 #include "config.h"
-
-#ifdef __INTEL_COMPILER
-#include <mathimf.h>
-#else
-#include <cmath>
-#endif
-
 #include "scalar_array.hpp"
 
 #include "data_types.hpp"
@@ -69,17 +62,76 @@
 #include <jemalloc/jemalloc.h>
 #endif
 
-#ifdef _MSC_VER
-// Intel compiler defines isnan in global namespace
-// MSVC defines _isnan
-# ifndef __INTEL_COMPILER
-#  define isnan _isnan
-# endif
-#elif __GLIBC__ == 2 && __GLIBC_MINOR__ < 23
-// https://sourceware.org/bugzilla/show_bug.cgi?id=19439
-#elif __cplusplus >= 201103L || !defined(__GLIBC__)
-using std::isnan;
-#endif
+namespace {
+struct EnvVar {
+  bool sumCriterion;
+  bool gessd;
+  bool mgsBlas3;
+  bool initPivot;
+  bool mgsAltPivot;
+  bool testOrtho;
+  EnvVar() {
+    sumCriterion = getenv("HMAT_SUM_CRITERION") != nullptr;
+    gessd = getenv("HMAT_GESDD") != nullptr;
+    mgsBlas3 = getenv("HMAT_MGS_BLAS3") != nullptr;
+    initPivot = getenv("HMAT_TRUNC_INITPIV") != nullptr;
+    mgsAltPivot = getenv("HMAT_MGS_ALTPIV") != nullptr;
+    testOrtho = getenv("HMAT_TEST_ORTHO") != nullptr;
+  }
+};
+const EnvVar env;
+
+/*! \brief Returns the number of singular values to keep.
+
+     The stop criterion is (assuming that the singular value
+     are in descending order):
+         sigma [k] / sigma[0] <epsilon
+     except if env. var. HMAT_SUM_CRITERION is set, in which case the criterion is:
+         sigma [k] / SUM (sigma) <epsilon
+
+     \param sigma table of singular values at least maxK elements.
+     \param epsilon tolerance.
+     \return int the number of singular values to keep.
+ */
+template <typename T>
+int findK(hmat::Vector<T> &sigma, double epsilon) {
+  assert(epsilon >= 0.);
+  double threshold_eigenvalue = 0.0;
+  if (env.sumCriterion) {
+    for (int i = 0; i < sigma.rows; i++) {
+      threshold_eigenvalue += sigma[i];
+    }
+  } else {
+    threshold_eigenvalue = sigma[0];
+  }
+  threshold_eigenvalue *= epsilon;
+  int i = 0;
+  for (i = 0; i < sigma.rows; i++) {
+    if (sigma[i] <= threshold_eigenvalue){
+      break;
+    }
+  }
+  return i;
+}
+
+struct SigmaPrinter {
+  bool enabled;
+  SigmaPrinter() {
+    enabled = getenv("HMAT_PRINT_SIGMA") != nullptr;
+  }
+  template <typename T> void print(const hmat::Vector<T> & sigma) const {
+    if(!enabled)
+      return;
+    // Use a buffer and printf for better output in multi-thread
+    std::stringstream buf;
+    for(int i = 0; i < sigma.rows; i++) {
+      buf << sigma[i] << " ";
+    }
+    printf("[SIGMA] %s\n", buf.str().c_str());
+  }
+};
+static SigmaPrinter sigmaPrinter;
+}
 
 namespace hmat {
 
@@ -93,6 +145,10 @@ Factorization convert_int_to_factorization(int t) {
         return Factorization::LDLT;
     case hmat_factorization_llt:
         return Factorization::LLT;
+    case hmat_factorization_hodlr:
+        return Factorization::HODLR;
+    case hmat_factorization_hodlrsym:
+        return Factorization::HODLRSYM;
     default:
         HMAT_ASSERT(false);
     }
@@ -108,6 +164,8 @@ int convert_factorization_to_int(Factorization f) {
         return hmat_factorization_ldlt;
     case Factorization::LLT:
         return hmat_factorization_llt;
+    case Factorization::HODLR:
+        return hmat_factorization_hodlr;
     default:
         HMAT_ASSERT(false);
     }
@@ -121,7 +179,9 @@ ScalarArray<T>::ScalarArray(T* _m, int _rows, int _cols, int _lda)
     lda = rows;
   }
   ownsFlag = true ;
+#ifdef HMAT_SCALAR_ARRAY_ORTHO
   is_ortho = (int*)calloc(1, sizeof(int));
+#endif
   assert(lda >= rows);
 }
 
@@ -129,6 +189,10 @@ template<typename T>
 ScalarArray<T>::ScalarArray(int _rows, int _cols, bool initzero)
   : ownsMemory(true), ownsFlag(true), rows(_rows), cols(_cols), lda(_rows) {
   size_t size = sizeof(T) * rows * cols;
+  if(size == 0) {
+    m = nullptr;
+    return;
+  }
   void * p;
 #ifdef HAVE_JEMALLOC
   p = initzero ? je_calloc(size, 1) : je_malloc(size);
@@ -136,8 +200,10 @@ ScalarArray<T>::ScalarArray(int _rows, int _cols, bool initzero)
   p = initzero ? calloc(size, 1) : malloc(size);
 #endif
   m = static_cast<T*>(p);
+#ifdef HMAT_SCALAR_ARRAY_ORTHO
   is_ortho = (int*)calloc(1, sizeof(int));
   setOrtho(initzero ? 1 : 0); // buffer filled with 0 is orthogonal
+#endif
   HMAT_ASSERT_MSG(m, "Trying to allocate %ldb of memory failed (rows=%d cols=%d sizeof(T)=%d)", size, rows, cols, sizeof(T));
   MemoryInstrumenter::instance().alloc(size, MemoryInstrumenter::FULL_MATRIX);
 }
@@ -153,10 +219,12 @@ template<typename T> ScalarArray<T>::~ScalarArray() {
 #endif
     m = NULL;
   }
+#ifdef HMAT_SCALAR_ARRAY_ORTHO
   if (ownsFlag) {
     free(is_ortho);
     is_ortho = NULL;
   }
+#endif
 }
 
 template<typename T> void ScalarArray<T>::resize(int col_num) {
@@ -181,7 +249,7 @@ template<typename T> void ScalarArray<T>::resize(int col_num) {
 
 template<typename T> void ScalarArray<T>::clear() {
   assert(lda == rows);
-  std::fill(m, m + ((size_t) rows) * cols, Constants<T>::zero);
+  std::fill(m, m + ((size_t) rows) * cols, 0);
   setOrtho(1); // we dont use ptr(): buffer filled with 0 is orthogonal
 }
 
@@ -200,7 +268,7 @@ template<typename T> size_t ScalarArray<T>::storedZeros() const {
 template<typename T> void ScalarArray<T>::scale(T alpha) {
   increment_flops(Multipliers<T>::mul * ((size_t) rows) * cols);
   if (lda == rows) {
-    if (alpha == Constants<T>::zero) {
+    if (alpha == T(0)) {
       this->clear();
     } else {
       // Warning: check for overflow
@@ -214,9 +282,9 @@ template<typename T> void ScalarArray<T>::scale(T alpha) {
     }
   } else {
     T* x = ptr();
-    if (alpha == Constants<T>::zero) {
+    if (alpha == T(0)) {
       for (int col = 0; col < cols; col++) {
-        std::fill(x, x + rows, Constants<T>::zero);
+        std::fill(x, x + rows, 0);
         x += lda;
       }
     } else {
@@ -226,7 +294,7 @@ template<typename T> void ScalarArray<T>::scale(T alpha) {
       }
     }
   }
-  if (alpha == Constants<T>::zero) setOrtho(1); // buffer filled with 0 is orthogonal
+  if (alpha == T(0)) setOrtho(1); // buffer filled with 0 is orthogonal
 }
 
 template<typename T> void ScalarArray<T>::transpose() {
@@ -422,7 +490,7 @@ void ScalarArray<T>::axpy(T alpha, const ScalarArray<T>* a) {
   size_t size = ((size_t) rows) * cols;
 
   increment_flops(Multipliers<T>::add * size
-		  + (alpha == Constants<T>::pone ? 0 : Multipliers<T>::mul * size));
+		  + (alpha == T(1) ? 0 : Multipliers<T>::mul * size));
   // Fast path
   if ((lda == rows) && (a->lda == a->rows) && (size < 1000000000)) {
     proxy_cblas::axpy(size, alpha, a->const_ptr(), 1, ptr(), 1);
@@ -437,7 +505,7 @@ void ScalarArray<T>::axpy(T alpha, const ScalarArray<T>* a) {
 template<typename T>
 double ScalarArray<T>::normSqr() const {
   size_t size = ((size_t) rows) * cols;
-  T result = Constants<T>::zero;
+  T result = 0;
 
   // Fast path
   if ((size < 1000000000) && (lda == rows)) {
@@ -543,7 +611,7 @@ template<typename T, typename std::enable_if<hmat::Types<T>::IS_REAL::value, T*>
 void checkNanSFINAE(const ScalarArray<T>* m) {
   for (int col = 0; col < m->cols; col++) {
     for (int row = 0; row < m->rows; row++) {
-      HMAT_ASSERT(!isnan(m->get(row, col)));
+      HMAT_ASSERT(std::isfinite(m->get(row, col)));
     }
   }
 }
@@ -552,8 +620,8 @@ template<typename T, typename std::enable_if<!hmat::Types<T>::IS_REAL::value, T*
 void checkNanSFINAE(const ScalarArray<T>* m) {
   for (int col = 0; col < m->cols; col++) {
     for (int row = 0; row < m->rows; row++) {
-      HMAT_ASSERT(!isnan(m->get(row, col).real()));
-      HMAT_ASSERT(!isnan(m->get(row, col).imag()));
+      HMAT_ASSERT(std::isfinite(m->get(row, col).real()));
+      HMAT_ASSERT(std::isfinite(m->get(row, col).imag()));
     }
   }
 }
@@ -566,7 +634,7 @@ void ScalarArray<T>::checkNan() const {
 template<typename T> bool ScalarArray<T>::isZero() const {
   for(int i = 0; i < rows; i++)
     for(int j = 0; j < cols; j++)
-      if (get(i, j) != Constants<T>::zero)
+      if (get(i, j) != T(0))
         return false;
   return true;
 }
@@ -640,7 +708,7 @@ void assertPositive(const T v, const int j, const char * const where) {
 
 template<typename T, typename std::enable_if<!hmat::Types<T>::IS_REAL::value, T*>::type = nullptr>
 void assertPositive(const T v, const int j, const char * const where) {
-    if(v == Constants<T>::zero)
+    if(v == T(0))
       throw InvalidDiagonalException<T>(v, j, where);
 }
 
@@ -673,7 +741,7 @@ void ScalarArray<T>::ldltDecomposition(Vector<T>& diagonal) {
         get(k,j) -= get(k,i) * v[i];
 
     for (int k = j+1; k < n; k++) {
-      if (v[j] == Constants<T>::zero)
+      if (v[j] == T(0))
         throw InvalidDiagonalException<T>(v[j], j, "ldltDecomposition");
       get(k,j) /= v[j];
     }
@@ -681,9 +749,9 @@ void ScalarArray<T>::ldltDecomposition(Vector<T>& diagonal) {
 
   for(int i = 0; i < n; i++) {
     diagonal[i] = get(i,i);
-    get(i,i) = Constants<T>::pone;
+    get(i,i) = 1;
     for (int j = i + 1; j < n; j++)
-      get(i,j) = Constants<T>::zero;
+      get(i,j) = 0;
   }
 
   delete[] v;
@@ -704,33 +772,33 @@ void ScalarArray<T>::lltDecomposition() {
   const size_t muls = n3 / 6 + n2 / 2 + n / 3;
   const size_t adds = n3 / 6 - n / 6;
     increment_flops(Multipliers<T>::add * adds + Multipliers<T>::mul * muls);
+  if(hmat::Types<T>::IS_REAL::value) {
+    // For real matrices, we can use the lapack version.
+    // (There is no L.Lt factorisation for complex matrices in lapack)
+    int info = proxy_lapack::potrf('L', rows, m, lda);
+    if(info != 0)
+      assertPositive(T(-1), info, "potrf");
+  } else {
+    for (int j = 0; j < n; j++) {
+      for (int k = 0; k < j; k++)
+        get(j,j) -= get(j,k) * get(j,k);
+      assertPositive(get(j, j), j, "lltDecomposition");
 
-  for (int j = 0; j < n; j++) {
-    for (int k = 0; k < j; k++)
-      get(j,j) -= get(j,k) * get(j,k);
-    assertPositive(get(j, j), j, "lltDecomposition");
+      get(j,j) = std::sqrt(get(j,j));
 
-    get(j,j) = std::sqrt(get(j,j));
+      for (int k = 0; k < j; k++)
+        for (int i = j+1; i < n; i++)
+          get(i,j) -= get(i,k) * get(j,k);
 
-    for (int k = 0; k < j; k++)
-      for (int i = j+1; i < n; i++)
-        get(i,j) -= get(i,k) * get(j,k);
-
-    for (int i = j+1; i < n; i++) {
-      get(i,j) /= get(j,j);
+      for (int i = j+1; i < n; i++) {
+        get(i,j) /= get(j,j);
+      }
     }
   }
 
-  // For real matrices, we could use the lapack version, could be faster
-  // (There is no L.Lt factorisation for complex matrices in lapack)
-  //    int info = proxy_lapack::potrf('L', this->rows(), this->data.m, this->data.lda);
-  //    if(info != 0)
-  //        // throw a pointer to be compliant with the Status class
-  //        throw hmat::LapackException("potrf", info);
-
   for (int j = 0; j < n; j++) {
         for(int i = 0; i < j; i++) {
-            get(i,j) = Constants<T>::zero;
+            get(i,j) = 0;
         }
     }
 }
@@ -758,16 +826,11 @@ inline char to_blas(const Diag diag) {
 // solve LX = (P ^ -1 B), which is done by ZLASWP with
 // the permutation. we used it just like in ZGETRS.
 template<typename T>
-void ScalarArray<T>::solveLowerTriangularLeft(ScalarArray<T>* x, const FactorizationData<T>& context, Diag diag, Uplo uplo) const {
-  {
-    const size_t _m = rows, _n = x->cols;
-    const size_t adds = _n * _m * (_m - 1) / 2;
-    const size_t muls = _n * _m * (_m + 1) / 2;
-    increment_flops(Multipliers<T>::add * adds + Multipliers<T>::mul * muls);
-  }
+void ScalarArray<T>::solveLowerTriangularLeft(ScalarArray<T>* x, const FactorizationData<T>& context,
+  Diag diag, Uplo uplo) const {
   if (context.algo == Factorization::LU && uplo == Uplo::LOWER)
     proxy_lapack::laswp(x->cols, x->ptr(), x->lda, 1, rows, context.data.pivots, 1);
-  proxy_cblas::trsm('L', to_blas(uplo), uplo == Uplo::LOWER ? 'N' : 'T', to_blas(diag), rows, x->cols, Constants<T>::pone, const_ptr(), lda, x->ptr(), x->lda);
+  x->trsm(Side::LEFT, uplo, uplo == Uplo::LOWER ? 'N' : 'T', diag, 1, this);
 }
 
 // The resolution of the upper triangular system does not need to
@@ -776,33 +839,15 @@ void ScalarArray<T>::solveLowerTriangularLeft(ScalarArray<T>* x, const Factoriza
 //  the matrix was factorized before.
 
 template<typename T>
-void ScalarArray<T>::solveUpperTriangularRight(ScalarArray<T>* x, const FactorizationData<T>& context, Diag diag, Uplo uplo) const {
-  // Void matrix
-  if (x->rows == 0 || x->cols == 0) return;
-
-  {
-    const size_t _m = rows, _n = x->cols;
-    const size_t adds = _n * _m * (_m - 1) / 2;
-    const size_t muls = _n * _m * (_m + 1) / 2;
-    increment_flops(Multipliers<T>::add * adds + Multipliers<T>::mul * muls);
-  }
-  proxy_cblas::trsm('R', to_blas(uplo), uplo == Uplo::LOWER ? 'T' : 'N', to_blas(diag),
-    x->rows, x->cols, Constants<T>::pone, const_ptr(), lda, x->ptr(), x->lda);
+void ScalarArray<T>::solveUpperTriangularRight(ScalarArray<T>* x, const FactorizationData<T>&,
+  Diag diag, Uplo uplo) const {
+  x->trsm(Side::RIGHT, uplo, uplo == Uplo::LOWER ? 'T' : 'N', diag, 1, this);
 }
 
 template<typename T>
-void ScalarArray<T>::solveUpperTriangularLeft(ScalarArray<T>* x, const FactorizationData<T>&, Diag diag, Uplo uplo) const {
-  // Void matrix
-  if (x->rows == 0 || x->cols == 0) return;
-
-  {
-    const size_t _m = rows, _n = x->cols;
-    const size_t adds = _n * _m * (_n - 1) / 2;
-    const size_t muls = _n * _m * (_n + 1) / 2;
-    increment_flops(Multipliers<T>::add * adds + Multipliers<T>::mul * muls);
-  }
-  proxy_cblas::trsm('L', to_blas(uplo), uplo == Uplo::LOWER ? 'T' : 'N', to_blas(diag),
-    x->rows, x->cols, Constants<T>::pone, const_ptr(), lda, x->ptr(), x->lda);
+void ScalarArray<T>::solveUpperTriangularLeft(ScalarArray<T>* x, const FactorizationData<T>&,
+  Diag diag, Uplo uplo) const {
+  x->trsm(Side::LEFT, uplo, uplo == Uplo::LOWER ? 'T' : 'N', diag, 1, this);
 }
 
 template<typename T>
@@ -849,59 +894,17 @@ void ScalarArray<T>::inverse() {
   }
   info = proxy_lapack::getrf(rows, cols, ptr(), lda, ipiv);
   HMAT_ASSERT(!info);
-  // We call it twice: the first time to know the optimal size of
-  // temporary arrays, and the second time for real calculation.
-  int workSize;
-  T workSize_req;
-  info = proxy_lapack::getri(rows, ptr(), lda, ipiv, &workSize_req, -1);
-  workSize = (int) real(workSize_req) + 1;
-  T* work = new T[workSize];
-  HMAT_ASSERT(work);
-  info = proxy_lapack::getri(rows, ptr(), lda, ipiv, work, workSize);
-  delete[] work;
+  info = proxy_lapack::getri(rows, ptr(), lda, ipiv);
   if (info)
     throw LapackException("getri", info);
   delete[] ipiv;
 }
 
-/*! \brief Returns the number of singular values to keep.
-
-     The stop criterion is (assuming that the singular value
-     are in descending order):
-         sigma [k] / sigma[0] <epsilon
-     except if env. var. HMAT_SUM_CRITERION is set, in which case the criterion is:
-         sigma [k] / SUM (sigma) <epsilon
-
-     \param sigma table of singular values at least maxK elements.
-     \param epsilon tolerance.
-     \return int the number of singular values to keep.
- */
-static int findK(Vector<double> &sigma, double epsilon) {
-  assert(epsilon >= 0.);
-  static char *useSumCriterion = getenv("HMAT_SUM_CRITERION");
-  double threshold_eigenvalue = 0.0;
-  if (useSumCriterion == NULL) {
-    threshold_eigenvalue = sigma[0];
-  } else {
-    for (int i = 0; i < sigma.rows; i++) {
-      threshold_eigenvalue += sigma[i];
-    }
-  }
-  threshold_eigenvalue *= epsilon;
-  int i = 0;
-  for (i = 0; i < sigma.rows; i++) {
-    if (sigma[i] <= threshold_eigenvalue){
-      break;
-    }
-  }
-  return i;
-}
-
 template<typename T> int ScalarArray<T>::truncatedSvdDecomposition(ScalarArray<T>** u, ScalarArray<T>** v, double epsilon, bool workAroundFailures) const {
-  Vector<double>* sigma = NULL;
+  Vector<typename Types<T>::real>* sigma = NULL;
 
   svdDecomposition(u, &sigma, v, workAroundFailures);
-
+  sigmaPrinter.print(*sigma);
   // Control of the approximation
   int newK = findK(*sigma, epsilon);
 
@@ -930,16 +933,19 @@ template<typename T> int ScalarArray<T>::truncatedSvdDecomposition(ScalarArray<T
   return newK;
 }
 
-  template<typename T> int ScalarArray<T>::svdDecomposition(ScalarArray<T>** u, Vector<double>** sigma, ScalarArray<T>** v, bool workAroundFailures) const {
+template <typename T>
+int ScalarArray<T>::svdDecomposition(ScalarArray<T> **u,
+                                     Vector<typename Types<T>::real> **sigma,
+                                     ScalarArray<T> **v,
+                                     bool workAroundFailures) const {
   DECLARE_CONTEXT;
-  static char * useGESDD = getenv("HMAT_GESDD");
   Timeline::Task t(Timeline::SVD, &rows, &cols);
   (void)t;
   // Allocate free space for U, S, V
   int p = std::min(rows, cols);
 
   *u = new ScalarArray<T>(rows, p, false);
-  *sigma = new Vector<double>(p);
+  *sigma = new Vector<typename Types<T>::real>(p);
   *v = new ScalarArray<T>(p, cols, false); // We create v in transposed shape (as expected by lapack zgesvd)
 
   // To be prepared for working around a failure in SVD, I must do a copy of 'this'
@@ -963,7 +969,7 @@ template<typename T> int ScalarArray<T>::truncatedSvdDecomposition(ScalarArray<T
   }
 
   try {
-    if(useGESDD)
+    if(env.gessd)
       info = sddCall(jobz, rows, cols, ptr(), lda, (*sigma)->ptr(), (*u)->ptr(),
                      (*u)->lda, (*v)->ptr(), (*v)->lda);
     else
@@ -978,13 +984,13 @@ template<typename T> int ScalarArray<T>::truncatedSvdDecomposition(ScalarArray<T
     // computation to proceed. Otherwise, I stop here.
     if (!workAroundFailures) throw;
 
-    printf("%s overriden...\n", e.what());
+    printf("%s overridden...\n", e.what());
     // If rows<cols, then p==rows, 'u' is square, 'v' has the dimensions of 'this'.
     // fake 'u' is identity, fake 'v' is 'this^T'
     if (rows<cols) {
       for (int i=0 ; i<rows ; i++)
         for (int j=0 ; j<p ; j++)
-          (*u)->get(i,j) = i==j ? Constants<T>::pone : Constants<T>::zero ;
+          (*u)->get(i,j) = i==j ? 1 : 0 ;
       (*u)->setOrtho(1);
       delete *v;
       *v = a ;
@@ -996,12 +1002,12 @@ template<typename T> int ScalarArray<T>::truncatedSvdDecomposition(ScalarArray<T
       *u = a ;
       for (int i=0 ; i<p ; i++)
         for (int j=0 ; j<cols ; j++)
-          (*v)->get(i,j) = i==j ? Constants<T>::pone : Constants<T>::zero ;
+          (*v)->get(i,j) = i==j ? 1 : 0 ;
       (*v)->setOrtho(1);
     }
     // Fake 'sigma' is all 1
     for (int i=0 ; i<p ; i++)
-      (**sigma)[i] = Constants<double>::pone ;
+      (**sigma)[i] = 1;
   }
 
   return info;
@@ -1017,27 +1023,26 @@ template<typename T> void ScalarArray<T>::orthoColumns(ScalarArray<T> *resultR,
     // Normalisation of column j
     Vector<T> aj(*this, j);
     resultR->get(j,j) = aj.norm();
-    T coef = Constants<T>::pone / resultR->get(j,j);
+    T coef = T(1) / resultR->get(j,j);
     aj.scale(coef);
   }
   // Remove the qj-component from vectors bk (k=initialPivot,...,n-1)
   if (initialPivot<cols) {
-    static char *useBlas3 = getenv("HMAT_MGS_BLAS3");
-    if (useBlas3) {
+    if (env.mgsBlas3) {
       ScalarArray<T> aJ(*this, 0, rows, 0, initialPivot); // All the columns of 'this' from 0 to 'initialPivot-1'
       ScalarArray<T> aJ_bK(*resultR, 0, initialPivot, initialPivot, cols-initialPivot); // In 'r': row '0' to 'initialPivot-1', all the columns after column 'initialPivot-1'
       // Compute in 1 operation all the scalar products between a_0,...,a_init-1 and a_init, ..., a_n-1
-      aJ_bK.gemm('C', 'N', Constants<T>::pone, &aJ, &bK, Constants<T>::zero);
+      aJ_bK.gemm('C', 'N', 1, &aJ, &bK, 0);
       // Update a_init, ..., a_n-1
-      bK.gemm('N', 'N', Constants<T>::mone, &aJ, &aJ_bK, Constants<T>::pone);
+      bK.gemm('N', 'N', -1, &aJ, &aJ_bK, 1);
     } else {
       for(int j = 0; j < initialPivot; ++j) {
         Vector<T> aj(*this, j);
         ScalarArray<T> aj_bK(*resultR, j, 1, initialPivot, cols-initialPivot); // In 'r': row 'j', all the columns after column 'initialPivot'
         // Compute in 1 operation all the scalar products between aj and a_firstcol, ..., a_n
-        aj_bK.gemm('C', 'N', Constants<T>::pone, &aj, &bK, Constants<T>::zero);
+        aj_bK.gemm('C', 'N', 1, &aj, &bK, 0);
         // Update a_firstcol, ..., a_n
-        bK.rankOneUpdateT(Constants<T>::mone, aj, aj_bK);
+        bK.rankOneUpdateT(-1, aj, aj_bK);
       }
     }
   } // if (initialPivot<cols)
@@ -1048,8 +1053,7 @@ template<typename T> void ScalarArray<T>::qrDecomposition(ScalarArray<T> *result
   Timeline::Task t(Timeline::QR, &rows, &cols, &initialPivot);
   (void)t;
 
-  static char *useInitPivot = getenv("HMAT_TRUNC_INITPIV");
-  if (!useInitPivot) initialPivot=0;
+  if (!env.initPivot) initialPivot=0;
   assert(initialPivot>=0 && initialPivot<=cols);
 
   ScalarArray<T> *bK = NULL, *restR = NULL, *a = this; // we need this because if initialPivot>0, we will run the end of the computation on a subset of 'this'
@@ -1074,18 +1078,7 @@ template<typename T> void ScalarArray<T>::qrDecomposition(ScalarArray<T> *result
     size_t additions = mm * n * n + (n * n * n) / 3 + 2 * mm * n - (n * n) / 2 + (5 * n) / 6;
     increment_flops(Multipliers<T>::mul * multiplications + Multipliers<T>::add * additions);
   }
-  int info;
-  int workSize;
-  T workSize_S;
-  // int info = LAPACKE_sgeqrf(LAPACK_COL_MAJOR, rows, cols, m, rows, *tau);
-  info = proxy_lapack::geqrf(a->rows, a->cols, a->ptr(), a->rows, tau, &workSize_S, -1);
-  HMAT_ASSERT(!info);
-  workSize = (int) hmat::real(workSize_S) + 1;
-  T* work = new T[workSize];// TODO Mettre dans la pile ??
-  HMAT_ASSERT(work) ;
-  info = proxy_lapack::geqrf(a->rows, a->cols, a->ptr(), a->rows, tau, work, workSize);
-  delete[] work;
-
+  int info = proxy_lapack::geqrf(a->rows, a->cols, a->ptr(), a->rows, tau);
   HMAT_ASSERT(!info);
 
   // Copy the 'r' factor in the upper part of resultR
@@ -1106,24 +1099,31 @@ template<typename T> void ScalarArray<T>::qrDecomposition(ScalarArray<T> *result
   return;
 }
 
-// aFull <- aFull.bTri^t with aFull=this and bTri upper triangular matrix
 template<typename T>
-void ScalarArray<T>::myTrmm(const ScalarArray<T>* bTri) {
+  void ScalarArray<T>::trmm(Side side, Uplo uplo,
+    char transA, Diag diag, T alpha, const ScalarArray<T> * a) {
   DECLARE_CONTEXT;
-  int mm = rows;
-  int n = rows;
-  T alpha = Constants<T>::pone;
-  const T *bData = bTri->const_ptr();
-  HMAT_ASSERT(bTri->lda == bTri->rows);
-  HMAT_ASSERT(lda == rows);
   {
-    size_t m_ = mm;
-    size_t nn = n;
-    size_t multiplications = m_ * nn  * (nn + 1) / 2;
-    size_t additions = m_ * nn  * (nn - 1) / 2;
+    size_t multiplications = (size_t)rows * cols * (cols + 1) / 2;
+    size_t additions = (size_t)rows * cols  * (cols - 1) / 2;
     increment_flops(Multipliers<T>::mul * multiplications + Multipliers<T>::add * additions);
   }
-  proxy_cblas::trmm('R', 'U', 'T', 'N', mm, n, alpha, bData, bTri->lda, ptr(), lda);
+  proxy_cblas::trmm(to_blas(side), to_blas(uplo), transA, to_blas(diag),
+    rows, cols, alpha, a->m, a->lda, this->m, this->lda);
+}
+
+template<typename T>
+  void ScalarArray<T>::trsm(Side side, Uplo uplo,
+    char transA, Diag diag, T alpha, const ScalarArray<T> * a) {
+  if (rows == 0 || cols == 0) return;
+  DECLARE_CONTEXT;
+  {
+    const size_t adds = cols * rows * (rows - 1) / 2;
+    const size_t muls = cols * rows * (rows + 1) / 2;
+    increment_flops(Multipliers<T>::add * adds + Multipliers<T>::mul * muls);
+  }
+  proxy_cblas::trsm(to_blas(side), to_blas(uplo), transA, to_blas(diag),
+    rows, cols, alpha, a->m, a->lda, this->m, this->lda);
 }
 
 template<typename T>
@@ -1132,9 +1132,6 @@ int ScalarArray<T>::productQ(char side, char trans, ScalarArray<T>* c) const {
   Timeline::Task t(Timeline::PRODUCTQ, &cols, &c->rows, &c->cols);
   (void)t;
   assert((side == 'L') ? rows == c->rows : rows == c->cols);
-  int info;
-  int workSize;
-  T workSize_req;
   {
     size_t _m = c->rows, _n = c->cols, _k = cols;
     size_t muls = 2 * _m * _n * _k - _n * _k * _k + 2 * _n * _k;
@@ -1149,24 +1146,8 @@ int ScalarArray<T>::productQ(char side, char trans, ScalarArray<T>* c) const {
   memcpy(tau, const_ptr(0, cols-1), sizeof(T)*std::min(rows, cols));
 
   // We don't use c->ptr() on purpose, because c->is_ortho is preserved here (Q is orthogonal)
-  info = proxy_lapack_convenience::or_un_mqr(side, trans, c->rows, c->cols, cols, const_ptr(), lda, tau, c->m, c->lda, &workSize_req, -1);
+  int info = proxy_lapack_convenience::or_un_mqr(side, trans, c->rows, c->cols, cols, const_ptr(), lda, tau, c->m, c->lda);
   HMAT_ASSERT(!info);
-  workSize = (int) hmat::real(workSize_req) + 1;
-
-  // If the previous call to 'or_un_mqr' does not give us a large enough work
-  // space size, we set the latter to the count of rows or columns of the matrix
-  // depending on the value of 'side'. 
-  if(side == 'L') {
-    workSize = workSize < c->rows ? c->rows : workSize;
-  } else if(side == 'R') {
-    workSize = workSize < c->cols ? c->cols : workSize;
-  }
-
-  T* work = new T[workSize];
-  HMAT_ASSERT(work);
-  info = proxy_lapack_convenience::or_un_mqr(side, trans, c->rows, c->cols, cols, const_ptr(), lda, tau, c->m, c->lda, work, workSize);
-  HMAT_ASSERT(!info);
-  delete[] work;
   return 0;
 }
 
@@ -1175,8 +1156,7 @@ template<typename T> int ScalarArray<T>::modifiedGramSchmidt(ScalarArray<T> *res
   Timeline::Task t(Timeline::MGS, &rows, &cols, &initialPivot);
   (void)t;
 
-  static char *useInitPivot = getenv("HMAT_MGS_INITPIV");
-  if (!useInitPivot) initialPivot=0;
+  if (!env.initPivot) initialPivot=0;
   assert(initialPivot>=0 && initialPivot<=cols);
 
 
@@ -1248,8 +1228,7 @@ template<typename T> int ScalarArray<T>::modifiedGramSchmidt(ScalarArray<T> *res
     int pivot = norm2.absoluteMaxIndex(j);
     double pivmax = norm2[pivot];
 
-    static char *newPivot = getenv("HMAT_MGS_ALTPIV");
-    if (newPivot) {
+    if (env.mgsAltPivot) {
       pivot = norm2_update.absoluteMaxIndex(j);
       pivmax = norm2_update[pivot];
       relative_epsilon = prec * prec;
@@ -1284,7 +1263,7 @@ template<typename T> int ScalarArray<T>::modifiedGramSchmidt(ScalarArray<T> *res
       // Normalisation of qj
       r.get(j,j) = sqrt(norm2[j]);
       Vector<T> aj(*this, j);
-      T coef = Constants<T>::pone / r.get(j,j);
+      T coef = T(1) / r.get(j,j);
       aj.scale(coef);
 
       // Remove the qj-component from vectors bk (k=j+1,...,n-1)
@@ -1293,9 +1272,9 @@ template<typename T> int ScalarArray<T>::modifiedGramSchmidt(ScalarArray<T> *res
         ScalarArray<T> bK(*this, 0, rows, firstcol, cols-firstcol); // All the columns of 'this' after column 'firstcol'
         ScalarArray<T> aj_bK(r, j, 1, firstcol, cols-firstcol); // In 'r': row 'j', all the columns after column 'firstcol'
         // Compute in 1 operation all the scalar products between aj and a_firstcol, ..., a_n
-        aj_bK.gemm('C', 'N', Constants<T>::pone, &aj, &bK, Constants<T>::zero);
+        aj_bK.gemm('C', 'N', 1, &aj, &bK, 0);
         // Update a_firstcol, ..., a_n
-        bK.rankOneUpdateT(Constants<T>::mone, aj, aj_bK);
+        bK.rankOneUpdateT(-1, aj, aj_bK);
         // Update the norms
         for(int k = firstcol; k < cols; ++k) {
           double rjk = std::abs(r.get(j,k));
@@ -1339,7 +1318,7 @@ void ScalarArray<T>::multiplyWithDiagOrDiagInv(const ScalarArray<T>* d, bool inv
     if (inverse) {
       ScalarArray<T> *d2 = new ScalarArray<T>(rows,1);
       for (int i = 0; i < rows; i++)
-        d2->get(i) = Constants<T>::pone / d->get(i);
+        d2->get(i) = T(1) / d->get(i);
       d = d2;
     }
     for (int j = 0; j < cols; j++) {
@@ -1350,14 +1329,14 @@ void ScalarArray<T>::multiplyWithDiagOrDiagInv(const ScalarArray<T>* d, bool inv
     if (inverse) delete(d);
   } else { // column j is multiplied by d[j] or 1/d[j]
     for (int j = 0; j < cols; j++) {
-      T diag_val = inverse ? Constants<T>::pone / d->get(j,0) : d->get(j);
+      T diag_val = inverse ? T(1) / d->get(j,0) : d->get(j);
       proxy_cblas::scal(rows, diag_val, ptr(0,j), 1);
     }
   }
 }
 
 template<typename T>
-void ScalarArray<T>::multiplyWithDiag(const ScalarArray<double>* d) {
+void ScalarArray<T>::multiplyWithDiag(const ScalarArray<typename hmat::Types<T>::real>* d) {
   assert(d);
   assert(cols <= d->rows); // d can be larger than needed
   assert(d->cols==1);
@@ -1367,14 +1346,29 @@ void ScalarArray<T>::multiplyWithDiag(const ScalarArray<double>* d) {
     increment_flops(Multipliers<T>::mul * _rows * _cols);
   }
   for (int j = 0; j < cols; j++) {
-    T diag_val = T(d->get(j));
-    proxy_cblas::scal(rows, diag_val, m+j*lda, 1); // We don't use ptr() on purpose, because is_ortho is preserved here
+    // We don't use ptr() on purpose, because is_ortho is preserved here
+    proxy_cblas::scal(rows, d->get(j), m+j*lda, 1);
   }
 }
 
+template<typename T> void ScalarArray<T>::addIdentity(T alpha) {
+  int n = std::min(this->rows, this->cols);
+  for(int i = 0; i < n; i++) {
+    get(i, i) += alpha;
+  }
+}
+
+template<typename T> typename Types<T>::dp ScalarArray<T>::diagonalProduct() const {
+  assert(rows == cols);
+  typename Types<T>::dp r = get(0,0);
+  for(int i = 1; i < rows; i++) {
+    r *= get(i,i);
+  }
+  return r;
+}
+
 template<typename T>
 bool ScalarArray<T>::testOrtho() const {
-  static char *test = getenv("HMAT_TEST_ORTHO");
   // code % 2 == 0 means we are in simple precision (real or complex)
   static double machine_accuracy = Constants<T>::code % 2 == 0 ? 1.19e-7 : 1.11e-16 ;
   static double test_accuracy = Constants<T>::code % 2 == 0 ? 1.e-3 : 1.e-7 ;
@@ -1383,14 +1377,14 @@ bool ScalarArray<T>::testOrtho() const {
   if (ref==0.) return true;
   ScalarArray<T> *sp = new ScalarArray<T>(cols, cols);
   // Compute the scalar product sp = X^H.X
-  sp->gemm('C', 'N', Constants<T>::pone, this, this, Constants<T>::zero);
+  sp->gemm('C', 'N', 1, this, this, 0);
   // Nullify the diagonal elements
   for (int i=0 ; i<cols ; i++)
-    sp->get(i,i) = Constants<T>::zero;
+    sp->get(i,i) = 0;
   // The norm of the rest should be below 'epsilon x norm of this' to have orthogonality and return true
   double res = sp->norm();
   delete sp;
-  if (test) {
+  if (env.testOrtho) {
     double ratio = res/ref/machine_accuracy/sqrt((double)rows);
     if (ratio > ratioMax) {
       ratioMax = ratio;
