@@ -42,6 +42,8 @@ enum class Factorization {
     LU = hmat_factorization_lu,
     LDLT = hmat_factorization_ldlt,
     LLT = hmat_factorization_llt,
+    HODLR = hmat_factorization_hodlr,
+    HODLRSYM = hmat_factorization_hodlrsym
 };
 
 enum class Side {
@@ -91,8 +93,10 @@ private:
 protected:
   /// Fortran style pointer (columnwise)
   T* m;
+#ifdef HMAT_SCALAR_ARRAY_ORTHO
   /*! flag for column orthogonality in m[] (it is a pointer because it is copied with m in subsets) */
   int *is_ortho;
+#endif
   /*! True if we own 'is_ortho' (there are cases where we own the flag and not the memory, with the constructor taking a 'T*' as input) */
   char ownsFlag:1;
 public:
@@ -110,7 +114,11 @@ public:
 
       \param d a ScalarArray
    */
-  ScalarArray(const ScalarArray& d) : ownsMemory(false), m(d.m), is_ortho(d.is_ortho), ownsFlag(false), rows(d.rows), cols(d.cols), lda(d.lda) {}
+  ScalarArray(const ScalarArray& d) : ownsMemory(false), m(d.m),
+#ifdef HMAT_SCALAR_ARRAY_ORTHO
+    is_ortho(d.is_ortho),
+#endif
+    ownsFlag(false), rows(d.rows), cols(d.cols), lda(d.lda) {}
   /** \brief Initialize the matrix with existing data.
 
       In this case the matrix doesn't own the data (the memory is not
@@ -133,7 +141,13 @@ public:
   ScalarArray(int _rows, int _cols, bool zeroinit=true);
   /** \brief Initialize the ScalarArray with subset of existing ScalarArray.
    */
-  ScalarArray(const ScalarArray& d, const int rowsOffset, const int rowsSize, const int colsOffset, const int colsSize): ownsMemory(false), m(d.m+rowsOffset+(size_t)colsOffset*d.lda), is_ortho(d.is_ortho), ownsFlag(false), rows(rowsSize), cols(colsSize), lda(d.lda){}
+  ScalarArray(const ScalarArray &d, const int rowsOffset, const int rowsSize,
+              const int colsOffset, const int colsSize)
+      : ownsMemory(false), m(d.m + rowsOffset + (size_t)colsOffset * d.lda),
+#ifdef HMAT_SCALAR_ARRAY_ORTHO
+        is_ortho(d.is_ortho),
+#endif
+        ownsFlag(false), rows(rowsSize), cols(colsSize), lda(d.lda) {}
 
   ~ScalarArray();
 
@@ -222,6 +236,10 @@ public:
 
     \return the matrix norm.
    */
+
+  /** Return the product of the diagonal values */
+  typename Types<T>::dp diagonalProduct() const;
+
   double norm_abt_Sqr(const ScalarArray<T> &b) const ;
 
   /*! \brief Compute dot product between a[i,*] and b[j,*]
@@ -367,7 +385,7 @@ public:
       \param workAroundFailures: handles the failures in lapack SVD (defaut is false)
       \return 0 for success, lapack error code otherwise.
    */
-  int svdDecomposition(ScalarArray<T>** u, Vector<double>** sigma, ScalarArray<T>** v, bool workAroundFailures=false) const;
+  int svdDecomposition(ScalarArray<T>** u, Vector<typename Types<T>::real>** sigma, ScalarArray<T>** v, bool workAroundFailures=false) const;
 
   /** Makes an truncated SVD of 'this' with accuracy 'epsilon'.
 
@@ -417,13 +435,11 @@ public:
   int productQ(char side, char trans, ScalarArray<T>* c) const;
 
 
-  /** Multiplication used in RkMatrix::truncate()
+  /** @brief Wrapper for Lapack trmm with B=this */
+  void trmm(Side side, Uplo uplo, char transA, Diag diag, T alpha, const ScalarArray<T> * a);
 
-       A B -> computing "AB^t" with A=this and B full upper triangular
-       (non-unitary diagonal)
-
-   */
-  void myTrmm(const ScalarArray<T>* bTri);
+  /** @brief Wrapper for Lapack trsm with B=this */
+  void trsm(Side side, Uplo uplo, char transA, Diag diag, T alpha, const ScalarArray<T> * a);
 
   /** modified Gram-Schmidt algorithm of A='this'
 
@@ -481,7 +497,7 @@ public:
 
      \param d  D
   */
-  void multiplyWithDiag(const ScalarArray<double>* d) ;
+  void multiplyWithDiag(const ScalarArray<typename Types<T>::real>* d) ;
 
   /*! \brief Computes if 'this' has orthogonal columns
 
@@ -492,16 +508,24 @@ public:
   /*! \brief Set orthogonality flag
    */
   inline void setOrtho(const int flag) const {
+#ifdef HMAT_SCALAR_ARRAY_ORTHO
     *is_ortho = flag;
     static char *test = getenv("HMAT_TEST_ORTHO");
     if (flag && test) assert(getOrtho() == testOrtho());
+#endif
   }
+
   /*! \brief Get orthogonality flag
    */
   inline int getOrtho() const {
+#ifdef HMAT_SCALAR_ARRAY_ORTHO
     return *is_ortho;
+#else
+    return 0;
+#endif
   }
 
+  void addIdentity(T alpha);
 };
 
   /*! \brief Templated Vector class = a ScalarArray with 1 column
